@@ -1,12 +1,15 @@
+from __future__ import annotations
+
+from collections.abc import Iterator
 from contextlib import suppress
-from typing import Any, Iterator, List, Optional
+from typing import Any
 
 import click
 from click import Command, Context
+from importlib_metadata import entry_points
 from natsort import natsort_keygen
 
 from . import __version__
-from ._compat import entry_points
 from .backends import Backend
 from .utils.click import MultiCommandX
 
@@ -28,13 +31,13 @@ class Cli(MultiCommandX):
                 assert isinstance(backend, Backend)
                 yield backend
 
-    def list_commands(self, ctx: Context) -> List[str]:
+    def list_commands(self, ctx: Context) -> list[str]:
         commands = set(self._ep_commands.names)
         for backend in self._iter_backends():
             commands.update(backend.algorithms_available)
         return sorted(commands, key=natsort_keygen())
 
-    def get_command(self, ctx: Context, name: str) -> Optional[Command]:
+    def get_command(self, ctx: Context, name: str) -> Command | None:
         with suppress(KeyError, ImportError):
             cmd = self._ep_commands[name].load()
             assert isinstance(cmd, Command)
