@@ -1,17 +1,21 @@
+from __future__ import annotations
+
 from importlib import import_module
-from typing import Set
+from typing import TYPE_CHECKING
 
-import Crypto  # noqa
+import Crypto  # noqa: F401
 
-from ..hasher import HashContext
 from . import Backend
 
+if TYPE_CHECKING:
+    from ..hasher import HashContext
+
 
 class PyCryptodomeBackend(Backend):
-    _algorithms = {"md2", "md4", "ripemd160"}
+    _algorithms = frozenset({"md2", "md4", "ripemd160"})
 
     @property
-    def algorithms_available(self) -> Set[str]:
+    def algorithms_available(self) -> frozenset[str]:
         return self._algorithms
 
     def load_ctx(self, name: str) -> HashContext:
