@@ -1,14 +1,14 @@
-import io
+from __future__ import annotations
+
 import os
-from os import PathLike
-from typing import Any, Dict, Optional, Protocol, Type, TypeVar, Union
+from pathlib import Path
+from typing import Any, Protocol
 
 from tqdm import tqdm
+from typing_extensions import Self
 
 from .utils.strxor import strxor
 
-__all__ = ["IsADirectory", "HashContext", "Hasher"]
-
 _CHUNKSIZE = 0x100000  # 1 MiB
 
 
@@ -16,24 +16,21 @@ class IsADirectory(OSError):
     """Raised by :meth:`Hasher.__call__`."""
 
 
-Self = TypeVar("Self", bound="HashContext")
-
-
 class HashContext(Protocol):
     """Typing for hash context."""
 
     @property
     def digest_size(self) -> int:
-        ...
+        """The size of the resulting hash in bytes."""
 
-    def update(self, data: bytes) -> None:
-        ...
+    def copy(self) -> Self:
+        """Return a copy the current hash context."""
 
     def digest(self) -> bytes:
-        ...
+        """Return the digest of the data passed so far."""
 
-    def copy(self: Self) -> Self:
-        ...
+    def update(self, data: bytes) -> None:
+        """Pass data to update the current hash context."""
 
 
 class Hasher:
@@ -57,9 +54,9 @@ class Hasher:
         self,
         ctx: HashContext,
         *,
-        chunksize: Optional[int] = None,
-        tqdm_args: Optional[Dict[str, Any]] = None,
-        tqdm_type: Optional[Type[tqdm]] = None,
+        chunksize: int | None = None,
+        tqdm_args: dict[str, Any] | None = None,
+        tqdm_type: type[tqdm] | None = None,
     ) -> None:
         if chunksize is None:
             chunksize = _CHUNKSIZE
@@ -94,17 +91,12 @@ class Hasher:
         self.tqdm_type = tqdm_type
 
     def __call__(
-        self,
-        path: "Union[str, PathLike[str]]",
-        start: Optional[int] = None,
-        stop: Optional[int] = None,
-        *,
-        dir_ok: bool = False,
+        self, path: str | Path, start: int | None = None, stop: int | None = None, *, dir_ok: bool = False
     ) -> bytes:
         """Return the hash value of a file or a directory.
 
         Parameters:
-            path (str | PathLike[str]):
+            path (str | Path):
                 The path of a file or a directory.
             start (int | None, default=None):
                 The start offset of the file or files in the directory.
@@ -122,39 +114,28 @@ class Hasher:
                 The hash value of the file or the directory.
         """
 
-        if start is not None and not isinstance(start, int):
-            tn = type(start).__name__
-            raise TypeError(f"start must be int or None, not {tn}")
-        if stop is not None and not isinstance(stop, int):
-            tn = type(stop).__name__
-            raise TypeError(f"stop must be int or None, not {tn}")
-
-        if os.path.isdir(path):
+        path = Path(path)
+        if path.is_dir():
             if dir_ok:
                 return self._hash_dir(path, start, stop)
-            raise IsADirectory(f"{path!r} is a directory")
+            raise IsADirectory(f"{str(path)!r} is a directory")
         return self._hash_file(path, start, stop)
 
-    def _hash_dir(
-        self, dirpath: "Union[str, PathLike[str]]", start: Optional[int] = None, stop: Optional[int] = None
-    ) -> bytes:
+    def _hash_dir(self, dirpath: Path, start: int | None = None, stop: int | None = None) -> bytes:
         # The initial hash value is all zeros.
         value = bytearray(self._ctx.digest_size)
-        with os.scandir(dirpath) as it:
-            for entry in it:
-                if entry.is_dir():
-                    other = self._hash_dir(entry.path, start, stop)
-                else:
-                    other = self._hash_file(entry.path, start, stop)
-                # Just XOR each byte string as the result of hashing.
-                strxor(value, other, value)
+        for entry in dirpath.iterdir():
+            if entry.is_dir():
+                other = self._hash_dir(entry, start, stop)
+            else:
+                other = self._hash_file(entry, start, stop)
+            # Just XOR each byte string as the result of hashing.
+            strxor(value, other, value)
         return bytes(value)
 
-    def _hash_file(
-        self, filepath: "Union[str, PathLike[str]]", start: Optional[int] = None, stop: Optional[int] = None
-    ) -> bytes:
+    def _hash_file(self, filepath: Path, start: int | None = None, stop: int | None = None) -> bytes:
         # Clamp `(start, stop)` to `(0, filesize)`.
-        filesize = os.path.getsize(filepath)
+        filesize = filepath.stat().st_size
         if start is None or start < 0:
             start = 0
         if stop is None or stop > filesize:
@@ -173,7 +154,7 @@ class Hasher:
 
         ctx = self._ctx.copy()
         with open(filepath, "rb") as f:
-            f.seek(start, io.SEEK_SET)
+            f.seek(start, os.SEEK_SET)
             with self.tqdm_type(total=total, **self.tqdm_args) as bar:
                 for _ in range(count):
                     chunk = f.read(chunksize)
