@@ -1,7 +1,12 @@
+from __future__ import annotations
+
+import abc
 import functools
 import os
 import sys
-from typing import Any, Callable, Iterable, Optional, TextIO, Tuple
+from collections.abc import Iterable
+from pathlib import Path
+from typing import Any, Callable, TextIO
 
 import click
 from click import Command
@@ -17,8 +22,8 @@ from .core import (
     generate_hash_line,
 )
 from .hasher import HashContext, Hasher
-from .utils.click import CommandX, PathWithSuffix
-from .utils.glob import glob_filters, sorted_path
+from .utils.click import CommandX
+from .utils.glob import auto_glob, glob_filters, sorted_path
 
 
 class ParseHashFileError(ValueError):
@@ -28,60 +33,79 @@ class ParseHashFileError(ValueError):
         self.lineno = lineno
 
 
-class Output:
-    """Determine the output mode and provide the output interface."""
+class Output(abc.ABC):
+    """Output interface for hash script."""
 
-    def __init__(
-        self, agg: Optional[str] = None, sep: Optional[bool] = None, null: Optional[bool] = None, sync: bool = False
-    ) -> None:
-        if (agg and sep) or (agg and null) or (sep and null):
-            raise ValueError("require exactly one argument")
+    @abc.abstractmethod
+    def close(self) -> None:
+        """Close output."""
 
-        # Use the null mode by default.
-        if not (agg or sep or null):
-            null = True
+    @abc.abstractmethod
+    def dump(self, hash_line: str, hash_path: str, path: str) -> None:
+        """Dump hash line to output."""
 
-        # Determine the output mode and dump method.
-        if agg:
-            self.agg_file = HashFileWriter(agg)
-            self._dump = self.output_agg
-        elif sep:
-            self._dump = self.output_sep
-        elif null:
-            self._dump = self.output_null
 
+class AggOutput(Output):
+    def __init__(self, filepath: str | Path, *, sync: bool = False) -> None:
+        self.hash_file = HashFileWriter(filepath)
         self.sync = sync
-        self.maxmtime = 0.0
+        self.maxt = 0
 
     def close(self) -> None:
-        try:
-            agg_file = self.agg_file
-        except AttributeError:
-            pass
-        else:
-            agg_file.close()
-            if self.sync:
-                os.utime(agg_file.name, (self.maxmtime, self.maxmtime))
+        self.hash_file.close()
+        if self.sync:
+            os.utime(self.hash_file.name, ns=(self.maxt, self.maxt))
 
     def dump(self, hash_line: str, hash_path: str, path: str) -> None:
-        self._dump(hash_line, hash_path, path)
-
-    def output_agg(self, hash_line: str, hash_path: str, path: str) -> None:
-        self.agg_file.write_hash_line(hash_line)
+        self.hash_file.write_hash_line(hash_line)
         if self.sync:
-            mtime = os.path.getmtime(path)
-            self.maxmtime = max(self.maxmtime, mtime)
+            self.maxt = max(os.stat(path).st_mtime_ns, self.maxt)
 
-    def output_sep(self, hash_line: str, hash_path: str, path: str) -> None:
-        with HashFileWriter(hash_path) as f:
-            f.write_hash_line(hash_line)
-        if self.sync:
-            mtime = os.path.getmtime(path)
-            os.utime(hash_path, (mtime, mtime))
 
-    def output_null(self, hash_line: str, hash_path: str, path: str) -> None:
+class SepOutput(Output):
+    def __init__(self, *, sync: bool = False) -> None:
+        self.sync = sync
+
+    def close(self) -> None:
         pass
 
+    def dump(self, hash_line: str, hash_path: str, path: str) -> None:
+        with HashFileWriter(hash_path) as hash_file:
+            hash_file.write_hash_line(hash_line)
+        if self.sync:
+            t = os.stat(path).st_mtime_ns
+            os.utime(hash_path, ns=(t, t))
+
+
+class NullOutput(Output):
+    def __init__(self) -> None:
+        pass
+
+    def close(self) -> None:
+        pass
+
+    def dump(self, hash_line: str, hash_path: str, path: str) -> None:
+        pass
+
+
+def create_output(
+    agg: str | None = None, sep: bool | None = None, null: bool | None = None, *, sync: bool = False
+) -> Output:
+    if (agg and sep) or (agg and null) or (sep and null):
+        raise ValueError("require exactly one argument")
+
+    # Use the null mode by default.
+    if not (agg or sep or null):
+        null = True
+
+    # Determine the output mode and dump method.
+    if agg:
+        return AggOutput(agg, sync=sync)
+    elif sep:
+        return SepOutput(sync=sync)
+    else:
+        return NullOutput()
+
 
 class Gethash:
     """Provide uniform interface for CLI scripts."""
@@ -93,14 +117,15 @@ class Gethash:
     glob_type: str
 
     inplace: bool
-    root: Optional[str]
+    root: str | None
 
-    start: Optional[int]
-    stop: Optional[int]
+    start: int | None
+    stop: int | None
     dir_ok: bool
 
     def __init__(self, ctx: HashContext, **kwargs: Any) -> None:
         self.ctx = ctx
+        self.auto = kwargs.pop("auto", False)
         self.sync = kwargs.pop("sync", False)
         self.suffix = kwargs.pop("suffix", ".sha")
 
@@ -118,7 +143,7 @@ class Gethash:
         agg = kwargs.pop("agg", None)
         sep = kwargs.pop("sep", None)
         null = kwargs.pop("null", None)
-        self.output = Output(agg, sep, null, sync=self.sync)
+        self.output = create_output(agg, sep, null, sync=self.sync)
 
         # Prepare arguments and construct the hash function.
         self.start = kwargs.pop("start", None)
@@ -138,7 +163,7 @@ class Gethash:
         else:
             self.generate_hash(files)
 
-    def __enter__(self) -> "Gethash":
+    def __enter__(self) -> Gethash:
         return self
 
     def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> None:
@@ -154,7 +179,7 @@ class Gethash:
                 hash_line = generate_hash_line(path, self.hash_function, root=root)
                 hash_path = path + self.suffix
                 self.output.dump(hash_line, hash_path, path)
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 self.echo_exception(path, e)
             else:
                 # The hash line already has a newline.
@@ -169,31 +194,33 @@ class Gethash:
                 hash_line = e.hash_line.rstrip("\n")
                 msg = f"[ERROR] invalid hash '{hash_line}' in '{hash_path}' at line {e.lineno}"
                 self.echo_error(msg, fg="white", bg="red")
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 self.echo_exception(hash_path, e)
 
     def _check_hash(self, hash_path: str) -> None:
-        maxmtime = 0.0
+        maxt = 0
         for i, hash_line in enumerate(HashFileReader(hash_path)):
             try:
                 root = self.check_root(hash_path)
                 path = check_hash_line(hash_line, self.hash_function, root=root)
-                maxmtime = max(maxmtime, os.path.getmtime(path))
+                maxt = max(os.stat(path).st_mtime_ns, maxt)
             except ParseHashLineError as e:
-                raise ParseHashFileError(e.hash_line, i)
+                raise ParseHashFileError(e.hash_line, i) from None
             except CheckHashLineError as e:
                 self.echo(f"[FAILURE] {e.path}", fg="red")
             else:
                 self.echo(f"[SUCCESS] {path}", fg="green")
         if self.sync:
-            os.utime(hash_path, (maxmtime, maxmtime))
+            os.utime(hash_path, ns=(maxt, maxt))
 
-    def check_root(self, path: str) -> Optional[str]:
+    def check_root(self, path: str) -> str | None:
         if self.inplace:
             return os.path.dirname(path)
         return self.root
 
     def glob_function(self, paths: Iterable[str]) -> Iterable[str]:
+        if self.auto:
+            return sorted_path(auto_glob(paths))
         return sorted_path(
             glob_filters(paths, mode=self.glob_mode, type=self.glob_type, recursive=True, user=True, vars=True)
         )
@@ -212,13 +239,13 @@ class Gethash:
         click.secho(msg, file=self.stderr, fg="red")
 
 
-def script_main(ctx: HashContext, files: Tuple[str, ...], **options: Any) -> None:
+def script_main(ctx: HashContext, files: tuple[str, ...], **options: Any) -> None:
     """Execute the body for the main function."""
 
     no_stdout = options.pop("no_stdout", False)
     no_stderr = options.pop("no_stderr", False)
-    stdout = open(os.devnull, "w") if no_stdout else sys.stdout  # noqa
-    stderr = open(os.devnull, "w") if no_stderr else sys.stderr  # noqa
+    stdout = open(os.devnull, "w") if no_stdout else sys.stdout  # noqa: SIM115
+    stderr = open(os.devnull, "w") if no_stderr else sys.stderr  # noqa: SIM115
 
     check = options.pop("check", False)
     with Gethash(ctx, stdout=stdout, stderr=stderr, **options) as gethash:
@@ -242,6 +269,7 @@ def gethashcli(command_name: str, display_name: str, **extras: Any) -> Callable[
 
         @click.command(command_name, cls=CommandX, context_settings=context_settings, no_args_is_help=True)
         @click.argument("files", nargs=-1)
+        @click.option("-a", "--auto", is_flag=True, help="Search files automatically")
         @click.option(
             "-c",
             "--check",
@@ -283,7 +311,7 @@ def gethashcli(command_name: str, display_name: str, **extras: Any) -> Callable[
         @output_mode.option(
             "-o",
             "--agg",
-            type=PathWithSuffix(suffix=suffix, dir_okay=False),
+            type=click.Path(dir_okay=False),
             help="Set the aggregate output file.",
         )
         @output_mode.option("-s", "--sep", is_flag=True, help="Separate output files.")
@@ -310,6 +338,8 @@ def gethashcli(command_name: str, display_name: str, **extras: Any) -> Callable[
         @functools.wraps(func)
         def wrapper(*args: Any, **kwargs: Any) -> Any:
             kwargs.setdefault("suffix", suffix)
+            if kwargs.get("auto", False):
+                kwargs.setdefault("files", (".",))
             return func(*args, **kwargs)
 
         return wrapper
